%Wrapper for the mathModel function for use w/ MindModeling
%By Chris Dancy @ Bucknell University
%For use with IGT-Open software
% Input:
%  inputMap - Contains IVs:
%   'updateChoice' - should be numbers to represent different
%    possible combinations of model choices:
%     ((DELta Update equation XOR DECay Update equation) AND 
%      (trial-dependent choice rule XOR trial-independent choice rule)
%     1 == DEL&t-d, 2 == DEL&t-i, 3 == DEC&t-d, 4 == DEC&t-i
%   FOR EVModel
%    'c' - 'Sensitivity' parameter (the higher, the more deterministic)
%    'weight' - 'Attention to loss' parameter
%    'a' - updating or recency parameter (this is currently alpha in the
%      actual mathModel file)
%   FOR PVModel
%    'c' - 'Sensitivity' parameter (the higher, the more deterministic)
%    'lambda' - 'Loss aversion' parameter
%    'alpha' - controls shape of utility function (this is currently pAlpha
%      in the actual mathModel file)
%    'a' - updating or recency parameter (this is currently alpha in the
%      actual mathModel file)

% This function is for use w/ the exptance valence model (and the params represent that).
function dvMap = mmEVModel(obj,inputMap) 
         
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1: Get Input Varibles
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    % IMPORTANT: The formal arguments here must match the IV names specified during job submission exactly, 
 
    %This isn't actually used yet, need to update to use!
    iv1 = values(inputMap,cellstr('updateChoice'));
    iv1Num = str2num(iv1{1});
    
    switch iv1Num
        case 1
            updateFun = 'delta';
            choiceFun = 't-d';
        case 2
            updateFun = 'delta';
            choiceFun = 't-i';
        case 3
            updateFun = 'delta';
            choiceFun = 't-d';
        case 4
            updateFun = 'decay';
            choiceFun = 't-i';
    end
         
    iv2 = values(inputMap,cellstr('c'));
    iv2Num = str2double(iv2{1});
    
    iv3 = values(inputMap,cellstr('weight'));
    iv3Num = str2double(iv3{1});
    
    iv4 = values(inputMap,cellstr('a'));
    iv4Num = str2double(iv4{1});
    
    try
        numTrials = values(inputMap, cellstr('nT'));
        numTrials = str2double(numTrials{1});
    catch
        numTrials = 100;
    end
    
    try
        numRuns = values(inputMap, cellstr('nR'));
        numRuns = str2double(numRuns{1});
    catch
        numRuns = 1000;
    end
   
    params = struct();
    params.c = iv2Num;
    params.uWeight = iv3Num;
    params.alpha = iv4Num;
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2: Initialize DVs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3: Calculate DVs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [maxNCards, decks] = PenaltyDist();
    blocks = mathModel(decks, maxNCards, params, numTrials, numRuns, ...
                       'e', updateFun, choiceFun);
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3: Store DVs in a container map
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     testKeySet = {'b1A','b2A','b3A','b1B','b2B','b3B',...
%             'b1C','b2C','b3C','b1D','b2D','b3D'};
    keySet={'b1A','b2A','b3A','b4A','b5A','b1B','b2B','b3B','b4B',...
            'b5B','b1C','b2C','b3C','b4C','b5C','b1D','b2D','b3D',...
            'b4D','b5D'};
    valueSet=blocks(:);
     
    % Return DVs as a container mapping of keys to values
    dvMap = containers.Map(keySet, valueSet);
end