%Wrapper for the mathModel function for use w/ MindModeling
%By Chris Dancy @ Bucknell University
%For use with IGT-Open software
% Input:
%  inputMap - Contains IVs:
%   'updateChoice' - should be numbers to represent different
%    possible combinations of model choices:
%     ((DELta Update equation OR DECay Update equation) AND 
%      (trial-dependent choice rule OR trial-independent choice rule)
%   FOR EVModel
%    'c' - 'Sensitivity' parameter (the higher, the more deterministic)
%    'weight' - 'Attention to loss' parameter
%    'a' - updating or recency parameter (this is currently alpha in the
%      actual mathModel file)
%   FOR PVModel
%    'c' - 'Sensitivity' parameter (the higher, the more deterministic)
%    'lambda' - 'Loss aversion' parameter
%    'alpha' - controls shape of utility function (this is currently pAlpha
%      in the actual mathModel file)
%    'a' - updating or recency parameter (this is currently alpha in the
%      actual mathModel file)

% This function is for use w/ the exptance valence model (and the params represent that).
function dvMap = mmPVModel(obj,inputMap) 
         
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1: Get Input Varibles
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    % IMPORTANT: The formal arguments here must match the IV names specified during job submission exactly, 
 
    %This isn't actually used yet, need to update to use!
    iv1 = values(inputMap,cellstr('updateChoice'));
    iv1Num = str2num(iv1{1});
    
    switch iv1Num
        case 1
            updateFun = 'delta';
            choiceFun = 't-d';
        case 2
            updateFun = 'delta';
            choiceFun = 't-i';
        case 3
            updateFun = 'delta';
            choiceFun = 't-d';
        case 4
            updateFun = 'decay';
            choiceFun = 't-i';
    end
         
    iv2 = values(inputMap,cellstr('c'));
    iv2Num = str2double(iv2{1});
    
    iv3 = values(inputMap,cellstr('lambda'));
    iv3Num = str2double(iv3{1});
    
    iv4 = values(inputMap,cellstr('alpha'));
    iv4Num = str2double(iv4{1});
    
    iv5 = values(inputMap,cellstr('a'));
    iv5Num = str2double(iv5{1});
   
    params = struct();
    params.c = iv2Num;
    params.lambda = iv3Num;
    params.pAlpha = iv4Num;
    params.alpha = iv5Num;
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2: Initialize DVs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3: Calculate DVs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [maxNCards, decks] = PenaltyDist();
    numTrials = 100;
    blocks = mathModel(decks, maxNCards, params, numTrials, 1000, ...
                       'p', updateFun, choiceFun);
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3: Store DVs in a container map
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
        keySet={'b1A','b2A','b3A','b4A','b5A','b1B','b2B','b3B','b4B',...
                'b5B','b1C','b2C','b3C','b4C','b5C','b1D','b2D','b3D',...
                'b4D','b5D'};
    valueSet=blocks(:);
     
    % Return DVs as a container mapping of keys to values
    dvMap = containers.Map(keySet, valueSet);
end