
function [numDataOut, maxComponents, myPort] = SetupEyeTracker(wPtr, comPort)
    %We want low output from the IOPort function
    IOPort('Verbosity', 0);
    
    res = Screen('Resolution', wPtr);
    
    %Number of bytes per piece of datum
    maxComponents = 2;
    
    %Total number of datum output by eye-tracker
    numDataOut = 28;
    
    %Other eye-tracker related vars
    sFreq = 60;
    baudRate = 57600;
    portName = comPort;
    inputBuffSize = 7200 * maxComponents * numDataOut * sFreq ;
    readTimeout = max(10 * 1/sFreq, 15);
    lineTerminator = 'CR/LF';
    portSettings = sprintf('BaudRate=%i InputBufferSize=%i Terminator=%i ReceiveTimeout=%f', baudRate, inputBuffSize, lineTerminator, readTimeout);
    
    %Open Port
    myPort = IOPort('OpenSerialPort', portName, portSettings);
    
    IOPort('Purge', myPort);
    
    % Configure background read
    asyncSetup = 'BlockingBackgroundRead=1 ReadFilterFlags=4';
    IOPort('ConfigureSerialPort', myPort, asyncSetup);
    
    %Start background read
    asyncSetup = sprintf('StartBackgroundRead=%i', maxComponents);
    IOPort('ConfigureSerialPort', myPort, asyncSetup);
    
    Screen('FillRect', wPtr, WhiteIndex(wPtr));
    Screen('DrawText', wPtr, 'R', res.width/2, res.height/2, BlackIndex(wPtr));
    Screen('Flip', wPtr);
    WaitSecs(2);
end

