%%****
   %This code was written by Christopher L. Dancy II while he was at Penn State,
   %University Park working on his PhD in IST (CogSci/AI focus).
   
   %Copyright 2014 Christopher L. Dancy II
%    This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%****

%%c.l.dancy@gmail.com is currently the most reliable way to contact Chris
%%Dancy

%This "clean" version of the code needs to be tested more, so if you find
%any error, please let me know and I likely can fix it fast   (or let
%me know if you actually have already fixed the problem

%%A future to do for this research is making it easy to modulate the number
%%of decks used (so we can better understand the processes used while
%%completing the IGT)

%I do not pretend that this code is currently optimized, but I hope to
%continuously improve it and if you have any suggestions, let me know.

% This code is the implementation of IGT from Bechara et al(2000) paper:
% "Characterization of the decision-making deficit of patients with ventromedial prefrontal cortex lesions"

%The visual stimuli used in the experiment that this was originally created
%for are from the IAPS and the numbers of the specific pictures used can be
%currently found in the dissertation that describes that work (this will
%likely be updated in the future to account for any related pubs). Chris
%Dancy's Dissertation can be found at http://www.etd.psu.edu/

%This code assumes you have the psychtoolbox library setup to run w/ Matlab
%(or Octave)

%This is written for a windows system, but should work with Linux also (If
%you use it and can't get it to work, please let me know and I will try to
%get you the older linux version I wrote previously).

%Modified waiting to use waitsecs command

%For parsing Serial Data the serial input (written from program to
%eye-tracker and relayed w/ data) code is as follows:
%0: Intertrial Break
%1-100: Trial Start/Num
 %trial 68 is instead marked with %250 (to avoid some ambiguities
 %associated with the ISCAN eye-tracker
%251:Selected deck marked
%252:Subliminal stimuli or flashed gray background presented
%253:Reward & Loss presented
%254:Intertrial break
%255:IGT Finished

%This function can also be used with the ACT-R JSON Module to communicate
%during the task. (The older Linux version does not currently contain this
%functionality)

% Group 1 is Negative images, 2 is Neutral, 3 is Positive
% mORf (Male or Female) 'm' - Male, 'f' - Female
% breaksLength = [Deck-phase length, Rew/Loss-phase length, Trial break
% length] OR [Deck-phase length, Rew/Loss-phase length, Trial break
% length, Stimulus-Presentation Length]
% maxTrial - Specifies the trial at which the experiment should halt before 
%  running (defaults to 101)
% 101)

%%To do:
%   -Think about loading all textures into struct at once at beginning
%of game and then just pass that struct to relevant functions
%   -Add more penalty function files
%   -Make it so user can specify number of decks
%   -Add GUI front-end
function igt_open_Time_Test(group, sid, savePath, mORf, penaltyFunStr, igtHomeDir, imageDir, breaksLength, comPort, simulation, maxTrial)
    deckAColor = '';
    deckBColor = '';
    deckCColor = '';
    deckDColor = '';
    
    penaltyFun = str2func(penaltyFunStr);
    
    if(exist('breaksLength', 'var') && (length(breaksLength) == 3 || length(breaksLength) == 4))
        dPLength = breaksLength(1);
        rlPLength = breaksLength(2);
        tBPLength = breaksLength(3);
        if( length(breaksLength) == 4 )
            sPLength = breaksLength(4);
        end
    end
    
    if(exist('simulation', 'var') && simulation)
        maxComponents = 0;
        myPort = -1;
        socketID = pnet('tcpsocket', 8088);
        modelCommID = -1;
        IGTMsg = '';
        disp('SIM!!!');
        %wait for IGT model to connect
        while( modelCommID == -1 )
            modelCommID = pnet(socketID, 'tcplisten');
        end
    else
        simulation = 0;
        modelCommID = -1;
    end
    KbName('UnifyKeyNames');
    cd(igtHomeDir);
    %give decks the penalty distribution
    [maxCards decks] = penaltyFun();
    
    subjFile = fopen(fullfile(savePath, strcat(mORf,'-', num2str(group), '-', num2str(sid),'.txt')), 'w');
    serialFile = fopen(fullfile(savePath, 'Serial-', strcat(mORf, '-', num2str(group), '-', num2str(sid),'.txt')), 'w');
    [wPtr, wRect] = InitScreen(simulation);
    Screen('Preference', 'VBLTimestampingMode', 1);
    global meanLum;
    meanLum = 0;
    
    if(~exist('simulation', 'var') || isempty(simulation) || ~simulation)
        if(exist('comPort', 'var') && comPort)
            [numDataOut maxComponents myPort] = SetupEyeTracker(wPtr, comPort);
        else
            maxComponents = 0;
            myPort = 0;
        end
    end
    
    if( myPort ~= -1 )
        %First we Calibrate
        %Calibrate(wPtr);
    
        %Give the user instructions
        %Instructions(wPtr);	

        %We practice before starting the game
        %PracticeRun(wPtr, decks);
    end
    
    %Check for a supplied image directory
    if(~exist('imageDir', 'var'))
        error('No image directory specified!!!');
    end
    
    if( strcmpi(mORf, 'm') )
        myStim = GetStimuli(fullfile(imageDir, 'stim', 'Male'));
    else
        myStim = GetStimuli(fullfile(imageDir, 'stim', 'Female'));
    end
    
    Screen('Preference', 'VBLTimestampingMode', 1);
    global sIndex;
    sIndex = 1;
    
    %specify order for image stimuli (randomize the starting image)
    stimOrder = randperm(length(myStim)-1);
    stimOrder = stimOrder + 1;
    acc_reward = 0;
    acc_punish = 0;
    
    if(~exist('maxTrial', 'var'))
        maxTrial = 101;
    end
    
    trial= 1;
    gameOrder = zeros(5,maxTrial); 
    respTimes = zeros(maxTrial,6);
    respTimesGetSecs = zeros(maxTrial);
    
    autoSelect = zeros(1, maxTrial); %used to record any automatic deck selections (due to time)
    if( myPort ~= -1 )
        %WaitForStart(wPtr);
        [deckParam deckAColor deckBColor deckCColor deckDColor tA tB tC tD] = ShowDecks(wPtr, decks, imageDir, '', maxCards);
        stimBoxParam = ShowStimBox(wPtr, deckParam, imageDir);
        ShowRewardBar(wPtr, acc_reward, acc_punish, trial);        
        
    elseif(exist('modelCommID', 'var') && ~isempty(modelCommID) && modelCommID ~= -1 )        
        %We will just update the display (in ACT-R w/out clearing it from
        %now on and thus we don't have to update in show decks everytime
        %just below the showdecks funtion in the trial loop below (this
        %follows w/ all the other display changes as well).
        %wait for model start message
        while( isempty(IGTMsg) || isempty(strfind(IGTMsg, 'model-run')) )
            IGTMsg = pnet(modelCommID, 'read', 'noblock');
        end

        IGTMsg = [];
        [deckParam deckAColor deckBColor deckCColor deckDColor tA tB tC tD] = ShowDecks(wPtr, decks, imageDir, modelCommID, maxCards);
        
        UpdateDisplayMsg(modelCommID, deckParam, [cellstr(deckAColor) ...
            cellstr(deckBColor) cellstr(deckCColor) cellstr(deckDColor)], 'IGT', 'true');
        
        SyncMsg(modelCommID);
        mpMsgCount = 0;
        while( isempty(IGTMsg) || ~isempty(strfind(IGTMsg, 'set-mp-time')) )
            IGTMsg = pnet(modelCommID, 'read', 'noblock');
            
            if( ~isempty(strfind(IGTMsg, 'set-mp-time')) )
                disp(IGTMsg);
                SyncMsg(modelCommID);
                mpMsgCount = mpMsgCount + 1;
            end
            
            %Once we encounter two set-mp-time Msgs in a row, break from this
            %loop
            if( mpMsgCount > 1 )
                break;
            end
        end
        while( isempty(IGTMsg) )
            IGTMsg = pnet(modelCommID, 'read', 'noblock');
        end
        disp(IGTMsg);
        SyncMsg(modelCommID);
        msgTimer = timer;
        set(msgTimer, 'TimerFcn', {@CheckSocketMsgs, num2str(modelCommID)})
        set(msgTimer, 'StartDelay', 0);
        set(msgTimer, 'ExecutionMode', 'fixedDelay');
        set(msgTimer, 'Period', 0.002);
        
        modelMsg = [];
        
        start(msgTimer);
        stimBoxParam = ShowStimBox(wPtr, deckParam, imageDir, modelCommID, ...
            [cellstr(deckAColor) cellstr(deckBColor) cellstr(deckCColor) cellstr(deckDColor)]);
        disp('here1');
        if(~isempty(pnet(modelCommID, 'read', 'noblock', 'view')))
            pnet(modelCommID, 'read', 'noblock');
            SyncMsg(modelCommID);
        end
        ShowRewardBar(wPtr, acc_reward, acc_punish, trial);
        
        testMsg = pnet(modelCommID, 'read', 'noblock', 'view');
        if(~isempty(testMsg))
            pnet(modelCommID, 'read', 'noblock');
            SyncMsg(modelCommID);
        end
    end
        startTime = clock();
        startTimeT = GetSecs();
        fprintf(subjFile, 'Start: %d %d %d %d %d %.6f - %.6f\n', startTime(1), startTime(2), ...
        startTime(3), startTime(4), startTime(5), startTime(6), startTimeT); %Record Start time to subject file
    
    
    
    if( myPort ~= -1 && myPort )
        fprintf(serialFile, 'Start: %d %d %d %d %d %.6f - %.6f\n', startTime(1), startTime(2), ...
            startTime(3), startTime(4), startTime(5), startTime(6), startTimeT);
    end
    
    Screen('Close', tA);
    Screen('Close', tB);
    Screen('Close', tC);
    Screen('Close', tD);
    %%%Actual experiment (minus the initial practice) starts here
    while( trial < maxTrial ) % iteration of card selection by subject
        
        %Write to eye-tracker so we know this data should be associated
        %with a break (intertrial or 20-trial block)
        if(myPort ~= -1 && myPort)
            IOPort('Write', myPort, uint8(254));
        end
        
        startTimeT = GetSecs();
        
		ShowRewardBar(wPtr, acc_reward, acc_punish, trial);
        
        if( myPort ~= -1 )
            [deckParam deckAColor deckBColor deckCColor deckDColor tA tB tC tD] = ShowDecks(wPtr, decks, imageDir, '', maxCards);
        else
            [deckParam deckAColor deckBColor deckCColor deckDColor tA tB tC tD] = ShowDecks(wPtr, decks, imageDir, modelCommID, maxCards);
            
            if(~isempty(pnet(modelCommID, 'read', 'noblock', 'view')))
                pnet(modelCommID, 'read', 'noblock');
                SyncMsg(modelCommID);
            end
        end
        
        %Write trial number to IOPort (for data collection purposes)
        % & avoid potential conflict in data analysis with trial #68 (we sub
        % #68 for #250)
        if( myPort ~= -1 && myPort )
            if(trial == 68)
                    IOPort('Write', myPort, uint8(250));
            else
                    IOPort('Write', myPort, uint8(trial));
            end
        end
        if(myPort ~= -1)
            stimBoxParam = ShowStimBox(wPtr, deckParam, imageDir);
            selectedDeck = num2str(mod(trial,4)+1);
            respTimeGetSecs = -1;
            respTime = -1;
            autoSelect(trial) = 1;
            %[selectedDeck, respTimeGetSecs, respTime, autoSelect(trial)] = GetResp(decks, myPort);
        else
            if(~isempty(pnet(modelCommID, 'read', 'noblock', 'view')))
                pnet(modelCommID, 'read', 'noblock');
                SyncMsg(modelCommID);
            end
            stimBoxParam = ShowStimBox(wPtr, deckParam, imageDir, modelCommID, ...
                [cellstr(deckAColor) cellstr(deckBColor) cellstr(deckCColor) cellstr(deckDColor)]);
            disp('here3');
            if(~isempty(pnet(modelCommID, 'read', 'noblock', 'view')))
                pnet(modelCommID, 'read', 'noblock');
                SyncMsg(modelCommID);
            end
            [selectedDeck, respTimeGetSecs, respTime, autoSelect(trial)] = GetResp(decks, myPort, modelCommID);
            
            if(~isempty(pnet(modelCommID, 'read', 'noblock', 'view')))
                pnet(modelCommID, 'read', 'noblock');
                SyncMsg(modelCommID);
            end
        end
        current_reward = 0;
        current_punish = 0;
        isRed = 0;

        if( strcmpi(selectedDeck, 'escape') )
            break;
        end
        
        if( strcmpi(selectedDeck, '\') && myPort ~= -1 )
            Calibrate(wPtr);
            continue;
        end

        if( strcmpi(selectedDeck, 'a') || strcmpi(selectedDeck, '1') || ...
                strcmpi(selectedDeck, '1!') )
            if (decks.index(1,1) <= maxCards)
                current_reward = decks.reward(1,decks.index(1,1));
                current_punish = decks.punish(1,decks.index(1,1));
                isRed = decks.redCard(1,decks.index(1,1));
                decks.index(1,1) = decks.index(1,1) + 1;
                selectedDeck = '1';
            else
                continue;
            end
        elseif( strcmpi(selectedDeck, 'b') || strcmpi(selectedDeck, '2') || ...
                strcmpi(selectedDeck, '2@') )
            if (decks.index(1,2) <= maxCards)
                current_reward = decks.reward(2,decks.index(1,2));
                current_punish = decks.punish(2,decks.index(1,2));
                isRed = decks.redCard(2,decks.index(1,2));
                decks.index(1,2) = decks.index(1,2) + 1;
                selectedDeck = '2';
            else
                continue;
            end
        elseif( strcmpi(selectedDeck, 'c') || strcmpi(selectedDeck, '3') || ...
                strcmpi(selectedDeck, '3#') )
            if (decks.index(1,3) <= maxCards)
                current_reward = decks.reward(3,decks.index(1,3));
                current_punish = decks.punish(3,decks.index(1,3));
                isRed = decks.redCard(3,decks.index(1,3));
                decks.index(1,3) = decks.index(1,3) + 1;
                selectedDeck = '3';
            else
                continue;
            end
        elseif( strcmpi(selectedDeck, 'd') || strcmpi(selectedDeck, '4') || ...
                strcmpi(selectedDeck, '4$') )
            if (decks.index(1,4) <= maxCards)
                current_reward = decks.reward(4,decks.index(1,4));
                current_punish = decks.punish(4,decks.index(1,4));
                isRed = decks.redCard(4,decks.index(1,4));
                decks.index(1,4) = decks.index(1,4) + 1;
                selectedDeck = '4';
            else
                continue;
            end
        end
        respTimes(trial) = respTime;
        respTimesGetSecs(trial) = respTime;
        [wT lT] = ShowMsg(wPtr, imageDir, current_reward, current_punish, selectedDeck, ...
            stimBoxParam, isRed, group, myStim, stimOrder(sIndex), ...
            myPort, modelCommID, [deckParam [cellstr(deckAColor) cellstr(deckBColor) cellstr(deckCColor) cellstr(deckDColor)]], mORf, dPLength, rlPLength, sPLength);
        acc_reward = acc_reward + current_reward;
        acc_punish = acc_punish + current_punish;
        gameOrder(:,trial) = [str2num(selectedDeck) current_reward acc_reward current_punish acc_punish];
        if( sIndex == 5 )
            sIndex = 1;
        else
            sIndex = sIndex+1;
        end
   
       trial=trial+ 1;
       ShowRewardBar(wPtr, acc_reward, acc_punish, trial);
       
       if(exist('modelCommID', 'var') && ~isempty(modelCommID) && modelCommID ~= -1 )
           UpdateDisplayMsg(modelCommID, deckParam, [cellstr(deckAColor) ...
                cellstr(deckBColor) cellstr(deckCColor) cellstr(deckDColor)], 'IGT', 'false', [], [], [], 1);
           disp('start next trial');
           %Wait for model's attention to shift to the message to start the new
           %trial
           SyncMsg(modelCommID);
           modelMsg = pnet(modelCommID, 'read', 'noblock');
           while(isempty(strfind(modelMsg, 'Next-Trial')))
              if(~isempty(modelMsg))
                  SyncMsg(modelCommID);
              end
              modelMsg = pnet(modelCommID, 'read', 'noblock');
           end
           SyncMsg(modelCommID);
           disp(trial);
       end
       
       %Cleanup and close image textures
       Screen('Close', tA);
       Screen('Close', tB);
       Screen('Close', tC);
       Screen('Close', tD);
       Screen('Close', wT);
       Screen('Close', lT);
    end
    if( myPort ~= -1 && myPort )
        %Signify the task has finished
        IOPort('Write', myPort, uint8(255));
    elseif(exist('modelCommID', 'var') && ~isempty(modelCommID) && modelCommID ~= -1 )
        SyncMsg(modelCommID);
        %Let the model know that the task is finished
        pnet(modelCommID, 'write', sprintf(char(strcat('{"model":"IGT",',...
                    '"method":"schedule-model-break","params":{"event":"(matlab-stop)"}}\n'))));
    end
    
    for i=1:(trial-1)
        fprintf(subjFile, '%d\t%.6f\t%.6f\t%d\t%d\t%d\t%d\t%d\n', i, respTimesGetSecs(i),respTimes(i), gameOrder(1,i), ...
                   gameOrder(2,i), gameOrder(3,i), gameOrder(4,i) , gameOrder(5,i));
    end
    
    fclose(subjFile);
    Finished(wPtr, myPort, modelCommID);
    if( myPort ~= -1 && myPort )
        GetEyeTrackerData(startTimeT, GetSecs(), myPort, serialFile, startTime, maxComponents);
    end
    Quit(myPort);
end
